// namespace, needed for Firefox
if ("undefined" == typeof(BFolders)) {
  var BFolders = {};
};

BFolders.handleLoginCaptured = function (login) {
	this.sendSaveCommand(login);
}

BFolders.sendQueryCommand = function(url) {
    // NOTE: !!! the server always de-escapes URL params so we must always escape !!!
	var pathAndQueryString = "query?url=" + encodeURIComponent(url);
	
	var responseCallback = function(response) {
		// check if login is found in the db
		if (!response.login)
			return;
	
		BFolders.doContentAutofillLogin(response.login);
	}
	
	this.sendCommand(pathAndQueryString, responseCallback, true);
}

BFolders.sendGeneratePasswordCommand = function(url) {
    // NOTE: !!! the server always de-escapes URL params so we must always escape !!!	
	var pathAndQueryString = "gen?url=" + encodeURIComponent(url);
	var responseCallback = function(response) {
		// the user may have canceled the password generation dialog
		if (response.password) {
			BFolders.doContentAutofillGeneratedPassword(response.password);
		}
	}
	
	this.sendCommand(pathAndQueryString, responseCallback, true);
}

BFolders.sendSaveCommand = function (login) {
	// NOTE: !!! the server always de-escapes URL params so we must always escape !!!
	
	// Note:
	// 
	// encodeURIComponent() does not do application/x-www-form-urlencoded but it intended for use with query strings
	// it does '+' => %2B and ' ' => %20
	// Java's URLDecoder.decode() handles encodeURIComponent() nicely 
	
	var params = 'url=' + encodeURIComponent(login.url);
	
	if (login.username) {
		params += '&username=' + encodeURIComponent(login.username)
	}
	
	if (login.password) {
		params += '&password=' + encodeURIComponent(login.password); 
	}		

	if (login.formKind) {
		params += '&formkind=' + encodeURIComponent(login.formKind);
	}
	
	var pathAndQueryString = "save?" + params;
	
	var responseCallback = function(response) {}
	
	// do not ask to retry - the server will decide what to do
	this.sendCommand(pathAndQueryString, responseCallback, false);
}

BFolders.sendCommand = function(pathAndQueryString, responseCallback, shouldAskToRetry) {
	// If the user re-triggers a previously failed command (without clicking Retry) and it succeeds
	// we must close any retry notifications.
	BFolders.closeAskToRetry();
	
	var request = this.createXMLHttpRequest();
	
	request.onload = function(aEvent) {
		
		if (aEvent.target.status != 200) {
			// it must be some incompatibility with the server - signal it with a null
			responseCallback(null);
			return;
		}
		
    	var text = aEvent.target.responseText;
   	
    	var response = JSON.parse(text);
    	
    	if (response.status == "OK") {	
    		responseCallback(response);
    	} else if(response.status == "E_NOT_LOGGED_IN") {
    		if (shouldAskToRetry) {
    			BFolders.askToRetry("B-Folders database is locked", pathAndQueryString, responseCallback);
    		}
    	} else if (response.status == "E_EDIT") {
    		if (shouldAskToRetry) {
    			BFolders.askToRetry("B-Folders is busy because an item is being edited.", pathAndQueryString, responseCallback);
    		}
    	} else if (response.status == "E_BUSY") {
    		if (shouldAskToRetry) {
    			BFolders.askToRetry("B-Folders is busy", pathAndQueryString, responseCallback);
    		}
    	}  	
	}
	
	request.onerror = function(aEvent) {
		if (shouldAskToRetry) {
			BFolders.askToRetry("B-Folders is not running", pathAndQueryString, responseCallback);
		}
	}
	
	request.open("GET", "http://127.0.0.1:8125/" + pathAndQueryString, true);
	request.send(); // we are using the GET method so no data to post
}