// namespace, needed for Firefox
if ("undefined" == typeof(BFolders)) {
  var BFolders = {};
};

BFolders.AUTOFILL_FLAG_ATTR = "bf_isautofilled";
BFolders.FORM_KIND_SIGN_UP = "signUp"; // part of the protocol with the server
BFolders.FORM_KIND_PASSWORD_CHANGE = "passwordChange"; // part of the protocol with the server
BFolders.FORM_KIND_LOGIN = "login"; // part of the protocol with the server

BFolders.getPasswordFields = function (form, skipEmpty) {
    // Locate the password fields in the form.
    var fields = [];
    
    for (var i = 0; i < form.elements.length; i++) {
        var e = form.elements[i];
        if (e.type != "password")
            continue;

        if (skipEmpty && !e.value)
            continue;
        
        fields[fields.length] = {index: i, element: e};
    }

    if (fields.length == 0 || fields.length > 3)
    	return null;

    return fields;
}

BFolders.getFieldsForAutofill = function (form) {
	var usernameField = null;
	
    var fields = this.getPasswordFields(form, false);
    
    if (!fields)
        return [null, null];
    
    for (var i = fields[0].index - 1; i >= 0; i--) {
        var e = form.elements[i];
        var fieldType = e.hasAttribute("type") ? e.getAttribute("type").toLowerCase() : e.type;
        if (fieldType == "text"  ||
            fieldType == "email" ||
            fieldType == "url"   ||
            fieldType == "tel"   ||
            fieldType == "number") {
            usernameField = e;
            break;
        }
    }
    
    return [usernameField, fields[0].element];
}


BFolders.captureLogin = function(form) {
    var fields = this.getPasswordFields(form, true); // skip empty fields

    // do not handle search and similar forms
    if (!fields)
        return null;
    
    var doc = form.ownerDocument;
    var login = {};
	login.url = doc.documentURI;
    
    // look for a username field, backwards from the first password field
    var usernameField = null;
    for (var i = fields[0].index - 1; i >= 0; i--) {
        var e = form.elements[i];
        var fieldType = e.hasAttribute("type") ? e.getAttribute("type").toLowerCase() : e.type;
        if (fieldType == "text"  ||
            fieldType == "email" ||
            fieldType == "url"   ||
            fieldType == "tel"   ||
            fieldType == "number") {
        	usernameField = e;
            break;
        }
    }
    
	if (usernameField) {
		login.username = usernameField.value;
	}
    
	// Guess the form type and which password field to save
    // !!! This is only a guess
    // for example twitter sign up form has one password field and may be confused for a login form
    var formKind;
    var passwordField;
    if (fields.length == 3) {
    	var p0 = fields[0].element.value;
    	var p1 = fields[1].element.value;
    	var p2 = fields[2].element.value;
    	
    	// the logic here is - if there are two equal passwords and one different,
    	// the equal passwords must be the new one 
    	
    	if (p0 == p1 && p1 == p2) {
    		// it's either a strange login form or a password change with all passwords equal which will not change the password
    		passwordField = fields[0].element;
    		formKind = this.FORM_KIND_LOGIN;
    	} else if (p0 != p1 && p1 == p2) {
    		// the configuration must be:
    		// old password
    		// new password
    		// new password confiramtion
    		formKind = this.FORM_KIND_PASSWORD_CHANGE;
    		passwordField = fields[1].element;
    	} else if (p0 == p1 && p1 != p2) {
    		// the configuration must be:
    		// new password
    		// new password confirmation
    		// old password
    		formKind = this.FORM_KIND_PASSWORD_CHANGE;
    		passwordField = fields[0].element;
    	} else if (p0 == p2 && p1 != p0) {
    		// the configuration must be:
    		// new password
    		// old password
    		// new password confirmation
    		formKind = this.FORM_KIND_PASSWORD_CHANGE;
    		passwordField = fields[0].element;
    	} else {
    		// it must be a wird form - the three passwords are different
    		formKind = this.FORM_KIND_PASSWORD_CHANGE;
    		passwordField = fields[0].element;
    	}
    } else if (fields.length == 2) {
    	var p0 = fields[0].element.value;
    	var p1 = fields[1].element.value;
    	if (p0 == p1) {
    		formKind = this.FORM_KIND_SIGN_UP;	
    		passwordField = fields[0].element;
    	} else {
    		formKind = this.FORM_KIND_PASSWORD_CHANGE;
    		// it must be a password change form without confirmation
    		// assume the second password is the new one
    		passwordField = fields[1].element; 
    	}
    } else {
    	formKind = this.FORM_KIND_LOGIN;
    	passwordField = fields[0].element; 
    }
    
    login.password = passwordField.value;
    login.formKind = formKind;
	
	// skip autofilled login forms
    // NOTE the user may autofill the old login in a password change form
	if (formKind == this.FORM_KIND_LOGIN) {
		// NOTE: we do set the flag when autofilling a generated password in registration forms
		if (usernameField && usernameField.hasAttribute(this.AUTOFILL_FLAG_ATTR)
			|| passwordField && passwordField.hasAttribute(this.AUTOFILL_FLAG_ATTR)) {
			return null;
		}
	}	
	
	return login;
}

//NOTE: called recursively
BFolders.autofillGeneratedPassword = function(password, win) {
	try {
		this.autofillGeneratedPasswordInDocument(password, win.document);
	} catch(e) {
		// cannot access the frame - cross domain access?
	}

	// RECURSION !!!
	for (var i = 0; i < win.frames.length; i++) {
		var childWin = win.frames[i]; 
		this.autofillGeneratedPassword(password, childWin);
	}
}

BFolders.autofillGeneratedPasswordInDocument = function(password, doc) {
	// a complex page may display both a registration form and a login widget on a top or side bar
	// so try to fill the registration form only

	for (var i = 0; i < doc.forms.length; i++) {
		var form = doc.forms[i];
		
		var passwordFields = this.getPasswordFields(form, false);
		if (!passwordFields)
			continue;
			
		// fill the last one or two of max three passowrd fields
		if (passwordFields.length == 1) {
			passwordFields[0].element.value = password;
		} else if (passwordFields.length == 2) {
			// both
			passwordFields[0].element.value = password;
			passwordFields[1].element.value = password;
		} else if (passwordFields.length == 3) {
			// the last 2 of 3
			passwordFields[1].element.value = password;
			passwordFields[2].element.value = password;
		}
	}
}


// NOTE: called recursively
BFolders.autofillLogin = function(login, win) {
	var wasFormSubmitted;
	try {
		wasFormSubmitted = this.autofillLoginInDocument(login, win.document);
		if (wasFormSubmitted)
			return true;
	} catch(e) {
		// cannot access the frame - cross domain access?
	}

	// RECURSION !!!
	for (var i = 0; i < win.frames.length; i++) {
		var childWin = win.frames[i]; 
		wasFormSubmitted = this.autofillLogin(login, childWin);
		if (wasFormSubmitted)
			return true;
	}
	
	return false;
}

// returns true if a form has been submitted and the autofilling other frames and 
// and forms must be cancelled
BFolders.autofillLoginInDocument = function(loginInfo, doc) {

	for (var i = 0; i < doc.forms.length; i++) {
		var form = doc.forms[i];
		
		var fields = this.getFieldsForAutofill(form);
		
		var usernameField = fields[0];
		var passwordField = fields[1];
		
		
		
        if (passwordField == null) {
            // nothing to autofill, hence must not submit even if the user wants so
            continue;
        }
        	
        // TODO consider if we should check the value of fields, some
        // web sites pre-fill those fields with hints (e.g. password, email) and
        // clear them with JS code
        	
		// NOTE: allow replacing autofilled username and password, but do not replace any value entered by the user
		
        if (loginInfo.password && (!passwordField.value || passwordField.hasAttribute(this.AUTOFILL_FLAG_ATTR))) {
        	passwordField.value = loginInfo.password;
			passwordField.setAttribute(this.AUTOFILL_FLAG_ATTR, "true");
        }
        
        if (loginInfo.username && usernameField && (!usernameField.value || usernameField.hasAttribute(this.AUTOFILL_FLAG_ATTR))) {
        	usernameField.value = loginInfo.username;
			usernameField.setAttribute(this.AUTOFILL_FLAG_ATTR, "true");
        }
        
        if (loginInfo.loginAid == "S") {
        	// login aid is autofill & submit
        	form.submit();
        
        	// one form is submitted, no need to process other forms
        	return true;
        }
	}
	
	return false;
}
